#
# Copyright (c) 2008-2018 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'

from heapq import merge
from itertools import chain, zip_longest
from random import shuffle

from zope.componentvocabulary.vocabulary import UtilityTerm, UtilityVocabulary
from zope.dublincore.interfaces import IZopeDublinCore

from pyams_content.shared.view.interfaces import CREATION_DATE_ORDER, FIRSTPUBLICATION_DATE_ORDER, IViewsMerger, \
    PUBLICATION_DATE_ORDER, UPDATE_DATE_ORDER, VIEWS_MERGERS_VOCABULARY
from pyams_utils.registry import utility_config
from pyams_utils.request import check_request
from pyams_utils.vocabulary import vocabulary_config
from pyams_workflow.interfaces import IWorkflowPublicationInfo

from pyams_content import _


@vocabulary_config(name=VIEWS_MERGERS_VOCABULARY)
class ViewsMergersVocabulary(UtilityVocabulary):
    """Views mergers vocabulary"""

    interface = IViewsMerger
    nameOnly = True

    def __init__(self, context, **kw):
        request = check_request()
        registry = request.registry
        translate = request.localizer.translate
        utils = [(name, translate(util.label))
                 for (name, util) in registry.getUtilitiesFor(self.interface)]
        self._terms = dict((title, UtilityTerm(name, title)) for name, title in utils)


CONCAT_VIEWS_MERGE_MODE = 'concat'


@utility_config(name=CONCAT_VIEWS_MERGE_MODE, provides=IViewsMerger)
class ViewsConcatenateMergeMode(object):
    """Views concatenate merge mode"""

    label = _("Concatenate views items in order")

    @classmethod
    def get_results(cls, views, context, ignore_cache=False, request=None):
        results = (view.get_results(context, ignore_cache=ignore_cache, request=request) for view in views)
        yield from chain(*results)


@utility_config(name='random', provides=IViewsMerger)
class ViewsRandomMergeMode(object):
    """Views random merge mode"""

    label = _("Extract items randomly")

    @classmethod
    def get_results(cls, views, context, ignore_cache=False, request=None):
        results = (view.get_results(context, ignore_cache=ignore_cache, request=request) for view in views)
        results = list(chain(*results))
        shuffle(results)
        yield from iter(results)


@utility_config(name='zip', provides=IViewsMerger)
class ViewsZipMergeMode(object):
    """Views zip merge mode"""

    label = _("Take items from views one by one, in views order")

    @classmethod
    def get_results(cls, views, context, ignore_cache=False, request=None):
        results = (view.get_results(context, ignore_cache=ignore_cache, request=request) for view in views)
        for array in zip_longest(*results):
            yield from filter(lambda x: x is not None, array)


@utility_config(name='zip_random', provides=IViewsMerger)
class ViewsRandomZipMergeMode(object):
    """Views random zip merge mode"""

    label = _("Take items from views one by one, in random order")

    @classmethod
    def get_results(cls, views, context, ignore_cache=False, request=None):
        results = [view.get_results(context, ignore_cache=ignore_cache, request=request) for view in views]
        shuffle(results)
        for array in zip_longest(*results):
            yield from filter(lambda x: x is not None, array)


class SortedMergeMode(object):
    """Sorted merge mode base class"""

    sort_index = None
    sort_key = None

    @classmethod
    def get_results(cls, views, context, ignore_cache=False, request=None):
        results = (sorted(view.get_results(context, ignore_cache=ignore_cache, request=request),
                          key=cls.sort_key, reverse=True)
                   for view in views)
        yield from merge(*results, key=cls.sort_key, reverse=True)


@utility_config(name='{0}.sort'.format(CREATION_DATE_ORDER), provides=IViewsMerger)
class CreationDateSortedMergeMode(SortedMergeMode):
    """Merge pre-sorted views by creation date"""

    label = _("Sort all results by creation date")
    sort_index = CREATION_DATE_ORDER
    sort_key = lambda x: IZopeDublinCore(x).created


@utility_config(name='{0}.sort'.format(UPDATE_DATE_ORDER), provides=IViewsMerger)
class UpdateDateSortedMergeMode(SortedMergeMode):
    """Merge pre-sorted views by last update date"""

    label = _("Sort all results by last update date")
    sort_index = UPDATE_DATE_ORDER
    sort_key = lambda x: IZopeDublinCore(x).modified


@utility_config(name='{0}.sort'.format(PUBLICATION_DATE_ORDER), provides=IViewsMerger)
class PublicationDateSortedMergeMode(SortedMergeMode):
    """Merge pre-sorted views by publication date"""

    label = _("Sort all results by current publication date")
    sort_index = PUBLICATION_DATE_ORDER
    sort_key = lambda x: IWorkflowPublicationInfo(x).publication_date


@utility_config(name='{0}.sort'.format(FIRSTPUBLICATION_DATE_ORDER), provides=IViewsMerger)
class FirstPublicationDateSortedMergeMode(SortedMergeMode):
    """Merge pre-sorted views by first publication date"""

    label = _("Sort all results by first publication date")
    sort_index = FIRSTPUBLICATION_DATE_ORDER
    sort_key = lambda x: IWorkflowPublicationInfo(x).first_publication_date
